#!/usr/bin/perl
#
#    Webmin module for Exim
#    Copyright (C) 2001 by Alexandre MATHIEU <mtlx@free.fr>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#
# dspl_msg.cgi

require 5;

use strict;
use integer;
use vars qw{%msg_actions %in %config %text};

require './spool-lib.pl';

ReadParse();

# Check the CGI 'msg_id' syntax and eventually perform a redirection.
my $msg_id = parse_CGI($in{msg_id}, 16, 'A-Za-z0-9\-');

defined $msg_id and $msg_id =~ /^(?:\w{6}-){2}\d{2}$/
  or redirect("/$::module_name/dspl_spool.cgi");

# Check if the message is always in the spool.
is_in_spool ($msg_id) or redirect("/$::module_name/dspl_spool.cgi"), exit();

# Webmin header
header($msg_id, '');
print '<hr>', "\n";

my $log = cast_exim (undef, "-Mvl $msg_id") or die ('Unable to fork');
LOG:
{
  my @lines = <$log>;
  last LOG unless @lines;
  last LOG if $lines[0] =~ /^Failed to open msglog file for /;

  map { HTML_entities($_) } @lines;

  print '<table border width="100%">', "\n";
  print "<tr $::tb><th>$text{msg_log}</th></tr>\n";
  print_tr (join '<br>', @lines);
  print '</table>', "\n";
  close $log or die 'Child died unexpectedly';
}

print <<HTML
<form action="proc_msg.cgi" method="post">
<input type="hidden" name="msg_id" value="$msg_id">
<br/>
HTML
;

if (is_frozen($msg_id))
  {
    delete $msg_actions{freeze}
  }
else
  {
    delete $msg_actions{thaw}
  }

foreach my $section ('view', 'proc', 'mdf')
  {
    print '<h3>', $text{"msg_$section"}, '</h3>', "\n";
    print '<table>', '<tr>', "\n";
    foreach my $action (keys %msg_actions)
      {
	next unless $msg_actions{$action}{sec} eq $section;

	print '<td>', "\n";
	print "<input type=\"submit\" name=\"$action\" value=\"", $text{"msg_$action"}, '"/>', "\n";

	if (exists $msg_actions{$action}{type})
	  {
	    $_ = $msg_actions{$action}{type};
	  TYPE:
	    {
	      /^text/ and do
		{
		  print "<input type=\"text\" name=\"$msg_actions{$action}{cgi}\" value=\"", get_msg_info ($msg_id, $msg_actions{$action}{value}), '">';
		  last TYPE
		};
	      /^select/ and do
		{
		  my $msg_info = get_msg_info ($msg_id, $msg_actions{$action}{value});
		  print "<select multiple size=\"@$msg_info\" name=\"$msg_actions{$action}{cgi}\">\n";
		  map { print "<option>$_</option>\n" } @$msg_info;
		  print '</select>';
		  last TYPE
		};
	    }
	  }
	print '&nbsp;</td>', "\n";
      }
    print "</tr></table>\n";
  }

print "<hr/>\n";
footer("/$::module_name/dspl_spool.cgi", $text{queue_title});
1;
