#!/usr/bin/perl
#
#    Webmin module for Exim
#    Copyright (C) 2001 by Alexandre MATHIEU <mtlx@free.fr>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#
# dspl_spool.cgi
# Display and search Exim's internal message queue

require 5;

use strict;
use integer;
use vars qw(%in %text @time_units @size_units);

my @sort_methods = ('gmt_date', 'size', 'time', 'sender', 'state');

require './spool-lib.pl';
header($text{queue_title}, '');
print "<hr/>\n";
my $update_time = time;
my $nb_of_msgs = store_spool ();
my $sort_method = 'default';

ReadParse ();

if (exists $in{sort})
  {
    $sort_method = parse_CGI ($in{sort}, 8, 'a-z_', @sort_methods);
    my @data;

    $_ = $sort_method;
  SORT: {
      /^size|time/ and do
	{
	  last SORT unless exists $in{v_min}
	    and exists $in{v_max}
	      and exists $in{u_min}
		and exists $in{u_max};
	  my (%max, %min);
	  my @units;
	  {
	    no strict 'refs';
	    @units = @{$sort_method.'_units'}
	  }
	  $max{unit} = rank (\@units, parse_CGI ($in{u_max}, 1, 'A-Za-z', @units));
	  $min{unit} = rank (\@units, parse_CGI ($in{u_min}, 1, 'A-Za-z', @units));
	  $max{value} = parse_CGI ($in{v_max}, 3, '0-9\._');
	  $min{value} = parse_CGI ($in{v_min}, 3, '0-9\._');

	  @data = (\%min, \%max);
	  last SORT
	};
      /^state/ and do {
	last SORT unless exists $in{is_frozen};
	@data = (parse_CGI ($in{is_frozen}, 1, '01'));
	last SORT;
      };
      /^sender/ and do {
	last SORT unless my $sender = parse_CGI ($in{sender}, 256, 'a-zA-Z0-9\.\-\@');
	@data = $sender;
	last SORT;
      }
    }
    sort_spool ($sort_method, @data) if @data;
  }
# Webmin headers


print '<b>', $text{queue_last_update}, ':', '&nbsp;' x 2, '</b>',  scalar gmtime $update_time, '<br/>' x 2, "\n";

if (not defined $nb_of_msgs)
  {
    print '<p><b>', $text{queue_no_messages}, '</b></p>', "\n";
    exit ();
  }

print_sort_banner ($sort_method);

search_by_form ($sort_method) if $sort_method=~/size|time|state|sender/;

print '<hr/>', "\n";
print '<table border width="100%">', "\n";
print_th (map { $text{"queue_$_"} } ('id', 'gmt_date', 'size', 'time', 'sender', 'frozen', 'recipients'));

foreach my $msg_id (@{get_spool ($sort_method)})
  {
    $msg_id =~ /^(\w{6})/ or next;
    my $gmt_date = gmtime base62_to_decimal ($1); # converts base 62 time to GMT

    my $size = get_msg_info ($msg_id, 'size');
    my $time = get_msg_info ($msg_id, 'time');

    print_tr ("<tt><a href=\"dspl_msg.cgi?msg_id=$msg_id\">$msg_id</a></tt>",
	      $gmt_date,
	      $$size{value}.'&nbsp;'.$text{"size_unit_$size_units[$$size{unit}]"},
	      $$time{value}.'&nbsp;'.$text{"time_unit_$time_units[$$time{unit}]"},
	      get_msg_info ($msg_id, 'sender'),
	      is_frozen ($msg_id)?$text{yes}:$text{no},
	      join "<br/>\n", @{get_msg_info ($msg_id, 'undelivered')},
	      map { "<font color=\"blue\">$_</font>" } @{get_msg_info ($msg_id, 'delivered')}
	     );
  }

print '</table>', "\n";

# Footer
print '<hr/>', "\n";
footer ('', $text{index_title});
1;

sub print_sort_banner
  {
    my $current_sort_method = shift;

    print '<br/>', "\n";
    print '<b>', $text{queue_sort_by}, ':&nbsp;&nbsp; ', '</b>';
    foreach my $sort_method (@sort_methods)
      {
	if ($sort_method eq $current_sort_method)
	  {
	    print '<b>', $text{"queue_$sort_method"}, '</b>'
	  }
	else
	  {
	    print('<a ', "href=\"dspl_spool.cgi?sort=$sort_method\">",
		  $text{'queue_'.$sort_method},
		  '</a>')
	  }
	print '&nbsp;'
      }
    print '<br/>' x2, "\n"
  }

sub search_by_form
  {
    my $method = shift;
    print '<form action="dspl_spool.cgi" method="post">', "\n";
    print "<input type=\"hidden\" name=\"sort\" value=\"$method\"/>\n";
    my @controls = ("<input type=\"submit\" value=\"$text{queue_search}\"/>\n");

  METHOD:
    {
      $method =~ /^time|size$/ and do
	{
	  my $min_max = get_min_max ($method);
	  my @current_units;
	  {
	    no strict 'refs';
	    @current_units = @{$method.'_units'};
	  }

	  my @units = @current_units[$$min_max{min}{unit}..$$min_max{max}{unit}];
	  foreach my $side ('max', 'min')
	    {
	      my $control = "<input type=\"text\" name=\"v_$side\" size=\"4\" maxlength=\"3\" value=\"$$min_max{$side}{value}\"/>\n";
	      $control .= "<select name=\"u_${side}\">";
	      foreach my $unit (reverse @units)
		{
		  $control .= "<option value=\"$unit\"";
		  $control .= $unit eq $current_units[$$min_max{$side}{unit}]?' selected ':' ';
		  $control .= '>'.$text{"${sort_method}_unit_$unit"}.'</option>'."\n"
		}
	      $control .= '</select>'."\n";
	      push @controls, $control;
	    }
	  last METHOD;
	};
      $method =~ /^state/ and do
	{
	  push @controls, '<input type="radio" name="is_frozen" value="1" checked />'."\n";
	  push @controls, '<input type="radio" name="is_frozen" value="0"/>'."\n";
	  last METHOD;
	};
      $method =~ /^sender/ and do
	{
	  push @controls, '<input type="text" name="sender"/>'."\n";
	  last METHOD;
	}
      }

    print '<b>', &text ("queue_search_by_$method", @controls), '</b>', "\n";
    print '</form>', "\n"
  }



