#!/usr/bin/perl
#
#    Webmin module for Exim
#    Copyright (C) 2001 by Alexandre MATHIEU
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#
# index.cgi

require 5;

use strict;
use integer;
use vars qw(%config %text $module_name $exim_path);

require './exim-lib.pl';

# Check for the exim executable
has_command ($exim_path) or display_error_page ('index_msgnoexe', $exim_path, $module_name), exit(0);

# Check for the configuration file
my $config_file = $config{exim_conf}?$config{exim_conf}:get_config_file_path ();
defined $config_file or display_error_page ('index_msgunknownconf', $module_name), exit(0);
{
  my @error_msg = check_config_file ($config_file);
  if (defined shift @error_msg)
    {
      my $error_msg = join '<br/>', @error_msg;
      display_error_page ('index_msgconfigerror', $config_file, $error_msg, $module_name);
      exit (0);
    }
}

# All is right, go on !

header ($text{index_title}, '', undef, 1, 1, undef,
	"<a href=\"mailto:$text{email}\">$text{author}</a><br/>
<a href=\"$text{homepage}\">$text{homepage}</a>");
print "<hr/>\n";

print '<h2>', $text{index_monitor}, '</h2>', "\n";

my @links = ('dspl_log.cgi', 'dspl_spool.cgi', 'dspl_stat.cgi');

my @titles = ($text{log_title}, $text{queue_title}, $text{stat_title});

my @icons = ('images/log.gif', 'images/spool.png', 'images/stat.png');

icons_table(\@links, \@titles, \@icons);

print '<br/>', "\n";
print '<hr/>', "\n";

footer('/', $text{index});
1;
